@@ -8,6 +8,7 @@ use crate::function::Function;
 use crate::operators::ArithmeticOp;
 use crate::operators::LogicalOp;
 use crate::operators::Op;
+use crate::query::Query;
 
 #[derive(Debug, Clone, PartialOrd, PartialEq, Eq, Hash, Serialize)]
 pub struct Expr {
@@ -21,10 +22,32 @@ pub struct Expr {
     pub function: Option<Function>,
     pub args: Option<Vec<Expr>>,
     pub val: Option<String>,
+    pub subquery: Option<Box<Query>>,
+    pub weight: i32,
 }
 
 impl Expr {
+    pub fn new() -> Expr {
+        Expr {
+            left: None,
+            arithmetic_op: None,
+            logical_op: None,
+            op: None,
+            right: None,
+            minus: false,
+            field: None,
+            function: None,
+            args: None,
+            val: None,
+            subquery: None,
+            weight: 0,
+        }
+    }
+    
     pub fn op(left: Expr, op: Op, right: Expr) -> Expr {
+        let left_weight = left.weight;
+        let right_weight = right.weight;
+
         Expr {
             left: Some(Box::new(left)),
             arithmetic_op: None,
@@ -36,10 +59,15 @@ impl Expr {
             function: None,
             args: None,
             val: None,
+            subquery: None,
+            weight: left_weight + right_weight,
         }
     }
 
     pub fn logical_op(left: Expr, logical_op: LogicalOp, right: Expr) -> Expr {
+        let left_weight = left.weight;
+        let right_weight = right.weight;
+
         Expr {
             left: Some(Box::new(left)),
             arithmetic_op: None,
@@ -51,10 +79,15 @@ impl Expr {
             function: None,
             args: None,
             val: None,
+            subquery: None,
+            weight: left_weight + right_weight,
         }
     }
 
     pub fn arithmetic_op(left: Expr, arithmetic_op: ArithmeticOp, right: Expr) -> Expr {
+        let left_weight = left.weight;
+        let right_weight = right.weight;
+
         Expr {
             left: Some(Box::new(left)),
             arithmetic_op: Some(arithmetic_op),
@@ -66,10 +99,14 @@ impl Expr {
             function: None,
             args: None,
             val: None,
+            subquery: None,
+            weight: left_weight + right_weight,
         }
     }
 
     pub fn field(field: Field) -> Expr {
+        let weight = field.get_weight();
+        
         Expr {
             left: None,
             arithmetic_op: None,
@@ -81,10 +118,14 @@ impl Expr {
             function: None,
             args: None,
             val: None,
+            subquery: None,
+            weight,
         }
     }
 
     pub fn function(function: Function) -> Expr {
+        let weight = function.get_weight();
+
         Expr {
             left: None,
             arithmetic_op: None,
@@ -96,10 +137,18 @@ impl Expr {
             function: Some(function),
             args: Some(vec![]),
             val: None,
+            subquery: None,
+            weight,
         }
     }
 
     pub fn function_left(function: Function, left: Option<Box<Expr>>) -> Expr {
+        let weight = function.get_weight();
+        let left_weight = match left {
+            Some(ref expr) => expr.weight,
+            None => 0,
+        };
+        
         Expr {
             left,
             arithmetic_op: None,
@@ -111,6 +160,8 @@ impl Expr {
             function: Some(function),
             args: Some(vec![]),
             val: None,
+            subquery: None,
+            weight: weight + left_weight,
         }
     }
 
@@ -126,8 +177,47 @@ impl Expr {
             function: None,
             args: None,
             val: Some(value),
+            subquery: None,
+            weight: 0,
         }
     }
+    
+    pub fn subquery(subquery: Query) -> Expr {
+        let weight = match subquery.expr {
+            Some(ref expr) => expr.weight,
+            None => 0,
+        };
+        
+        Expr {
+            left: None,
+            arithmetic_op: None,
+            logical_op: None,
+            op: None,
+            right: None,
+            minus: false,
+            field: None,
+            function: None,
+            args: None,
+            val: None,
+            subquery: Some(Box::new(subquery)),
+            weight,
+        }
+    }
+    
+    pub fn add_left(&mut self, left: Expr) {
+        let left_weight = left.weight;
+        self.left = Some(Box::new(left));
+        self.weight += left_weight;
+    }
+    
+    pub fn set_args(&mut self, args: Vec<Expr>) {
+        let mut args_weight = 0;
+        for arg in &args {
+            args_weight += arg.weight;
+        }
+        self.args = Some(args);
+        self.weight += args_weight;
+    }
 
     pub fn has_aggregate_function(&self) -> bool {
         if let Some(ref left) = self.left {
@@ -291,3 +381,74 @@ impl Display for Expr {
         Ok(())
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::field::Field;
+    use crate::function::Function;
+
+    #[test]
+    fn test_weight() {
+        let expr = Expr::field(Field::Name);
+        assert_eq!(expr.weight, 0);
+        
+        let expr = Expr::field(Field::Accessed);
+        assert_eq!(expr.weight, 1);
+        
+        let expr = Expr::function(Function::Concat);
+        assert_eq!(expr.weight, 0);
+        
+        let expr = Expr::function(Function::Contains);
+        assert_eq!(expr.weight, 1024);
+        
+        let expr = Expr::function_left(Function::Contains, Some(Box::new(Expr::value("foo".to_string()))));
+        assert_eq!(expr.weight, 1024);
+        
+        let expr = Expr::logical_op(
+            Expr::op(
+                Expr::field(Field::Size),
+                Op::Gt,
+                Expr::value(String::from("456")),
+            ),
+            LogicalOp::Or,
+            Expr::op(
+                Expr::field(Field::FormattedSize),
+                Op::Lte,
+                Expr::value(String::from("758")),
+            ),
+        );
+        assert_eq!(expr.weight, 2);
+        
+        let expr = Expr::logical_op(
+            Expr::logical_op(
+                Expr::op(
+                    Expr::field(Field::Name),
+                    Op::Ne,
+                    Expr::value(String::from("123")),
+                ),
+                LogicalOp::And,
+                Expr::logical_op(
+                    Expr::op(
+                        Expr::field(Field::Size),
+                        Op::Gt,
+                        Expr::value(String::from("456")),
+                    ),
+                    LogicalOp::Or,
+                    Expr::op(
+                        Expr::field(Field::FormattedSize),
+                        Op::Lte,
+                        Expr::value(String::from("758")),
+                    ),
+                ),
+            ),
+            LogicalOp::Or,
+            Expr::op(
+                Expr::field(Field::Name),
+                Op::Eq,
+                Expr::value(String::from("xxx")),
+            ),
+        );
+        assert_eq!(expr.weight, 2);
+    }
+}
\ No newline at end of file
